<?php declare(strict_types=1);

namespace Drupal\burnafter;

use Drupal\encrypt\EncryptServiceInterface;
use Drupal\encrypt\EncryptionProfileInterface;
use Drupal\encrypt\EncryptionProfileManagerInterface;
use Drupal\encrypt\Exception\EncryptException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Provides a helper service for managing burnafter entities.
 */
final class BurnAfterService implements BurnAfterServiceInterface {

  /**
   * Constructs a BurnAfterService object.
   *
   * @param \Drupal\encrypt\EncryptServiceInterface $encryptService
   * @param \Drupal\encrypt\EncryptionProfileManagerInterface $encryptProfileManager
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Psr\Log\LoggerInterface $logger
   */
  public function __construct(
    private readonly EncryptServiceInterface $encryptService,
    private readonly EncryptionProfileManagerInterface $encryptProfileManager,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly LoggerInterface $logger
  ) {}
  
  /**
   * {@inheritdoc}
   */
  public function shouldEncrypt(): bool {
    return $this->configFactory->get('burnafter.settings')->get('enable_encryption') === true;
  }

  /**
   * {@inheritdoc}
   */
  public function encrypt(string $data): string {
    try {
      return $this->encryptService->encrypt($data, $this->getEncryptionProfile());
    }
    catch (\RuntimeException $e) {
      $this->logger->error('Encryption profile error: @message', ['@message' => $e->getMessage()]);
    }
    catch (EncryptException $e) {
      $this->logger->error('Encryption failed: @message', ['@message' => $e->getMessage()]);
      throw $e;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function decrypt(string $data): string {
    try {
      return $this->encryptService->decrypt($data, $this->getEncryptionProfile());
    }
    catch (\RuntimeException $e) {
      $this->logger->error('Encryption profile error: @message', ['@message' => $e->getMessage()]);
    }
    catch (EncryptException $e) {
      $this->logger->error('Encryption failed: @message', ['@message' => $e->getMessage()]);
      throw $e;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteExpired(): void {
    $deleted = 0;

    foreach ($this
      ->entityTypeManager
      ->getStorage('burnafter')
      ->loadMultiple() as $burnafter) {

      if ($burnafter->shouldDelete()) {
        $burnafter->delete();
        ++$deleted;
      }
    }

    if ($deleted > 0) {
      $this->logger->info('Burned @num entities.', ['@num' => $deleted]);
    }
  }

  /**
   * Get the selected encryption profile to use for
   * encrypting burnafter entity content.
   *
   * @return \Drupal\encrypt\EncryptionProfileInterface.
   */
  private function getEncryptionProfile(): EncryptionProfileInterface {
    $profile_id = $this->configFactory->get('burnafter.settings')->get('encryption_profile');
    if (!$profile_id) {
      throw new \RuntimeException('Encryption profile not set.');
    }

    $profile = $this->encryptProfileManager->getEncryptionProfile($profile_id);
    if (!$profile) {
      throw new \RuntimeException('Encryption profile not found.');
    }

    $errors = $profile->validate();
    if (!empty($errors)) {
      throw new EncryptException(implode(';', $errors));
    }

    return $profile;
  }

}

// vim: expandtab ts=2 sw=2

