<?php declare(strict_types=1);

namespace Drupal\burnafter\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\encrypt\Exception\EncryptException;

/**
 * Defines the burnafter entity.
 *
 * @ContentEntityType(
 *   id = "burnafter",
 *   label = @Translation("BurnAfter"),
 *   handlers = {
 *     "form" = {
 *       "default" = "Drupal\burnafter\Form\BurnAfterForm"
 *     },
 *     "access" = "Drupal\burnafter\Entity\BurnAfterEntityAccessHandler"
 *   },
 *   base_table = "burnafter",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid"
 *   }
 * )
 */
final class BurnAfterEntity extends ContentEntityBase implements BurnAfterEntityInterface {

  /**
   * {@inheritdoc}
   */
  public function shouldDelete(): bool {
    $now = time();

    $expire_at = (int) $this->get('expire_at')->value;
    $view_count = (int) $this->get('view_count')->value;
    $max_views = (int) $this->get('max_views')->value;

    return $expire_at <= $now || $view_count >= $max_views;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage): void {
    parent::preSave($storage);

    $burnafter = \Drupal::service('burnafter');
    if ($this->isNew() && $burnafter->shouldEncrypt()) {
      try {
        $content = $this->get('content')->value;
        $this->set('content', $burnafter->encrypt($content));
      }
      catch (EncryptException $e) {
        throw new EntityStorageException("Failed to save: {$e->getMessage()}", 0, $e);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function toUrl($rel = NULL, array $options = []): Url {
    switch ($rel) {
    case 'add':
      $route = 'burnafter.add';
      break;

    case 'settings':
      $route = 'burnafter.settings';
      break;

    case 'view':
    default:
      $route = 'burnafter.view';
      $params = ['burnafter' => $this->uuid()];
      break;
    }

    return new Url($route, $params ?? [], $options);
  }

  /**
   * {@inheritdoc}
   */
  public static function getDefaultExpireAt(): int {
    // Current time + 30 minutes from now.
    return time() + (30 * 60);
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['content'] = BaseFieldDefinition::create('string_long')
      ->setLabel(new TranslatableMarkup('Content'))
      ->setRequired(true)
      ->setDisplayOptions('form', [
        'type' => 'string_textarea',
        'weight' => 0,
      ]);

    $fields['view_count'] = BaseFieldDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('View count'))
      ->setDefaultValue(0)
      ->setReadOnly(true);

    $fields['max_views'] = BaseFieldDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Maximum views'))
      ->setDefaultValue(1)
      ->setRequired(true)
      ->setSetting('min', 1)
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => 5,
      ]);

    $fields['expire_at'] = BaseFieldDefinition::create('timestamp')
      ->setLabel(new TranslatableMarkup('Expiration time'))
      ->setRequired(true)
      ->setDefaultValueCallback('Drupal\burnafter\Entity\BurnAfterEntity::getDefaultExpireAt')
      ->setDescription(new TranslatableMarkup('This entity will self-destruct by this date and time unless maximum views is reached first.'))
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 10,
      ])
      ->addConstraint('NotPastDate');

    return $fields;
  }

}

// vim: expandtab ts=2 sw=2

