<?php declare(strict_types=1);

namespace Drupal\burnafter\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\encrypt\EncryptionProfileManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Settings form for BurnAfter module.
 */
final class BurnAfterSettingsForm extends ConfigFormBase {

  /**
   * Constructs a BurnAfterSettingsForm object.
   *
   * @param \Drupal\encrypt\EncryptionProfileManagerInterface $encryptionProfileManager.
   */
  public function __construct(
    private readonly EncryptionProfileManagerInterface $encryptionProfileManager,
    private readonly ModuleHandlerInterface $moduleHandler
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('encrypt.encryption_profile.manager'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['burnafter.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'burnafter_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('burnafter.settings');

    if ($this->moduleHandler->moduleExists('encrypt')) {
      $form['enable_encryption'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable encryption'),
        '#default_value' => $config->get('enable_encryption'),
        '#description' => $this->t('If checked, the content field will be encrypted in the database.'),
      ];

      $profiles = [];
      foreach ($this->encryptionProfileManager->getAllEncryptionProfiles() as $profile) {
        $profiles[$profile->id()] = $profile->label();
      }

      $form['encryption_profile'] = [
        '#type' => 'select',
        '#title' => $this->t('Encryption profile'),
        '#options' => $profiles,
        '#default_value' => $config->get('encryption_profile'),
        '#states' => [
          'visible' => [
            ':input[name="enable_encryption"]' => ['checked' => true],
           ],
          'required' => [
            ':input[name="enable_encryption"]' => ['checked' => true],
           ],
        ],
        '#description' => $this->t('Select the encryption profile to use.'),
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this
      ->config('burnafter.settings')
      ->set('enable_encryption', $form_state->getValue('enable_encryption'))
      ->set('encryption_profile', $form_state->getValue('encryption_profile'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}

// vim: expandtab ts=2 sw=2

